package math;
//package scoot.math.linalg;

import java.io.Serializable;
//import scoot.math.random.*;

/** 
  Implements an m by n matrix with double floating point elements.
  Works with Vec for vector math.  Vec is different from a 1xn or nx1 matrix in that  
  it offers other methods.
  
  <P>
  Indices are 0-based.
  
  <P>
  <B>NOTE:</B> Any of these methods may throw an ArrayIndexOutOfBoundsException. 
  Since this is a runtime exception, you need not catch it if you check bounds.
  
  <P>
  Some of the operators also throw runtime exceptions for dimension
  mismatch or inplace multiplication errors.  If you write your code
  carefully, you needn't worry about these either; hence, they are
  declared runtime exceptions.  
  
  <P>
  SingularMatrixException must be handled.
    
  @see aries.math.linalg.Vec
  @see aries.math.linalg.LUDState
  @see SingularMatrixException
  @see DimensionMismatchException
  @see InPlaceMatrixMultException */

  public class Matrix implements Cloneable, Serializable 
  {
    /* 
       Rep
     */
    // package scope for efficiency
    double[][] rep;
    int rows, cols;
    
    public final static double TINY = 1.0e-11;
    public final static double CLOSE_ENOUGH_TO_ZERO = 1.0e-11;

    /*
      Constructors
      */

    public Matrix() {}

    protected void updateSize()
    {
      rows = rep.length;
      if (rows > 0)
	cols = rep[0].length;
      else
	cols = 0;
    }

    public double[][] getValues() { return rep; }

    public void setValues(double[][] a) 
    { 
      rep = a;
      updateSize();
    }
    

    /** 
      @param rows the number of rows 
      @param cols the numbers of columns 
      */
    public Matrix(int rows, int cols)
    {
      rep = new double[rows][cols];
      updateSize();
    }
    
    /** 
      Create the matrix as a copy of another matrix
      */
    public Matrix(Matrix m)
    {
      rep = copyRep(m.rep);
      updateSize();
    }
    
    /** 
      Create the matrix from a 2-dimensional array of floats, where
      the size of the matrix is defined as double[rows][columns].  
      <B>Note:</B> makes a copy of the array, does <em>not</em> store the reference.
      */
    public Matrix(double[][] a)
    {
      rep = copyRep(a);
      updateSize();
    }


    /** Create the matrix from the given diagonal elements. */
    public Matrix(double[] a)
    {
      rep = new double[a.length][a.length];
      for (int i = 0; i < a.length; i++)
	rep[i][i] = a[i];
      updateSize();
    }
    

    /**
      Clone and copy the matrix and return the copy.
      */
    public Matrix copy()
    {
      return (Matrix) clone();
      /*
      Matrix m = new Matrix();
      m.rep = Matrix.copyRep(rep);
      m.rows = rep.length;
      m.cols = rep[0].length;
      return m;
      */
    }
    
    public void fromMatrix(Matrix m)
    {
      rep = copyRep(m.rep);
      updateSize();
    }
    
    public void copyFrom(Matrix A, int[] i1, int[] i2)
    {
      int n1 = i1.length;
      int n2 = i2.length;
      double[] row;
      for (int i = 0; i < n1; i++)
	{
	  row = A.rep[i1[i]];
	  for (int j = 0; j < n2; j++)
	    rep[i][j] = row[i2[j]];
	}
    }


    public void copyFrom(Matrix from)
    {
      if (rows != from.rows || cols != from.cols)
	throw new DimensionMismatchException("copyFrom: different size matrices");
      int i,j;
      for(i=0; i< rows; i++)
	for(j=0; j< cols; j++)
	  rep[i][j] = from.rep[i][j];
    }

    public Object clone()
    {
      try
	{
	  Matrix m = (Matrix)super.clone();
	  m.rep = Matrix.copyRep(rep);
	  m.updateSize();
	  return m;
	}
      catch (CloneNotSupportedException e)
	{
	  e.printStackTrace();
	  return null;
	}
    }
    
    /**
      Observers
      ***/
    
    public final int numRows()
    {
      return rows;
    }
  
    public final int numColumns()
    {
      return cols;
    }

    /**
      return the (i,j) element of the matrix. 
      @exception java.lang.ArrayIndexOutOfBoundsException  if index out of range
      */
    public final double ref(int i, int j)
    {
      return rep[i][j];
    }
    
    /** 
      same as ref(i,j) 
      @exception java.lang.ArrayIndexOutOfBoundsException  if index out of range
      */
    public final double get(int i, int j)
    {
      return rep[i][j];
    }
  
    /** return the elements in a 2-dimensional double array, where
      double[rows][columns] is the layout of elements.  
      <B>NOTE:</B> Returns a COPY of the data, not a reference.  Changes to the array 
      returned will not side-effect this object */
    public final double[][] toArray() { return copyRep(rep); }
    
    public String toString()
    {
      if (rows == 0) return "empty";
      String s = "";
      for (int i=0; i<rows; i++)
      {
	Vec v = new Vec(rep[i]);
	for (int j=0; j < v.size()-1; j++)
	{
	  s += Format.format(v.get(j), true) + " ";
	}
	s += Format.format(v.get(v.size()-1), true);
	if (i < rows-1)
	  s += "\n";
      }
      return s;
    }

    /*********************
      Mutators
      ********************/

    /** set the value of the (i,j) element to v.
      @param i row index
      @param j column index
      @param v value to set it to.
      @exception java.lang.ArrayIndexOutOfBoundsException  if index out of range
      */
    public final void set(int i, int j, double v)
    {
      rep[i][j] = v;
    }
  
    /** set this matrix's value to be the elements of the other
      matrix. Matrices' dimensions must match.
      @param m source matrix. */
    public final void set(Matrix m) throws DimensionMismatchException
    {
      if ((cols != m.cols) ||
	  (rows != m.rows) ||
	  (cols == 0) ||
	  (rows == 0))
	{
	  throw new DimensionMismatchException(
	    "Matrices' dimensions must match for in-place set. This matrix " +
	    "was " + rows + "x" + cols + ", argument given is " +
	    m.rows + "x" + m.cols
	  );
	}
      int i,j;
      for (i=0; i<rows; i++)
	for (j=0; j<cols; j++)
	  rep[i][j] = m.rep[i][j];
    }

    /**
      Mutate myself into my transpose.
      <B>Note:</B> MUTATES object, does not return a new matrix.
      To create a new Matrix, use the static method transpose.
      @see Matrix#transpose(Matrix)
      @see Matrix#makeTranspose()
      */
    public final void transpose()
    {
      int i,j;
      if (rows == cols)
	{
	  for(i=0; i<rows; i++)
	    for(j=0; j < i; j++)
	      {
		double t = rep[i][j];
		rep[i][j] = rep[j][i];
		rep[j][i] = t;
	      }
	}
      else
	{
	  // new matrix.
	  double[][] new_rep = new double[cols][rows];
	  for (i=0; i<rows; i++)
	    for(j=0; j<cols; j++)
	      new_rep[j][i] = rep[i][j];
	  rep = new_rep;
	  updateSize();
	}
    }

    /**
      Return a new matrix which a transpose of this matrix.
      Does <en>not</em> change this.
    */
    public final Matrix makeTranspose()
    {
      Matrix m = new Matrix(this);
      m.transpose();
      return m;
    }

    /**
       Resize the matrix to rows, cols.  Truncates if smaller, zero pads if larger. Does no work if the size is correct, so can be used without checking.  */
    public void resize(int rows, int cols)
    {
      if (rows == rows &&
	  cols == cols)
	return;
	  
      double[][] new_rep = new double[rows][cols];
      for (int i=0; i<rows; i++)
	{
	  for (int j=0; j<cols; j++)
	  {
	    if (i < rows &&
		j < cols)
	    {
	      new_rep[i][j] = rep[i][j];
	    }
	    else
	    {
	      new_rep[i][j] = 0.0;
	    }
	  }
	}
      rep = new_rep;
      updateSize();
    }

    /**
      Turns elements "close" to zero into zeros.
      Specifically, 
      For all elements e, if abs(e) < CLOSE_ENOUGH_TO_ZERO, set e = 0.0.
      
      <P>
      <B>MUTATES</B> this.
      */
    public void roundZeros()
    {
      int i,j;
      for(i=0; i<rows; i++)
	for(j=0; j<cols; j++)
	  if (Math.abs(rep[i][j]) < CLOSE_ENOUGH_TO_ZERO) rep[i][j] = 0.0;
    }

    /**
      Scale each element in the matrix by the scalar d.
      @param d the scalar to scale by.
      <P><B>MUTATES</B> this.
      */
    public final void scale(double d)
    {
      int i,j;
      for(i=0; i < rows; i++)
	for(j=0; j < cols; j++)
	  rep[i][j] *= d;
    }

    public final void zero()
    {
      int i,j;
      for(i=0; i < rows; i++)
	for(j=0; j < cols; j++)
	  rep[i][j] = 0;
    }
  
    public final void add(int i, int j, double d)
    {
      rep[i][j] += d;
    }

    public final void sub(int i, int j, double d)
    {
      rep[i][j] -= d;
    }


    /**
      Turn this into the identity matrix.
      If non-square, turns all elements (i,j) : i==j to 1.0 and all others to 0.0
      <P><B>MUTATES</B> this.
      */
    public final void identity()
    {
      scale(0.0);

      int i = 0;
      while (i < rows &&
	     i < cols)
	{
	  rep[i][i] = 1.0;
	  i++;
	}
    }
    
    public static Matrix identity(int d)
    {
      Matrix I = new Matrix(d,d);
      I.identity();
      return I;
    }

    /** 
	Make this into a diagonal matrix using d as the diagonal entries.  The matrix will be resized to d.size() x d.size() if it is not already.  All off-diagonal entries will be set to 0.0.
    */
    public void setDiagonal(Vec d)
    {
      int n = d.size();
      resize(n,n);
      int i,j;
      for (i=0; i<n; i++)
	{
	  for (j=0; j<n; j++)
	    {
	      if (i == j)
		rep[i][j] = d.get(i);
	      else 
		rep[i][j] = 0.0;
	    }
	}
    }

    public final void scaleAdd(double scale, Matrix a)
    {
      int i,j;
      for(i=0; i < rows; i++)
	for(j=0; j < cols; j++)
	  rep[i][j] += scale*a.rep[i][j];
    }

    /**
      element-wise matrix addition.
      <B>SAFE:</B>  Does not effect either this or b.
      There is static add method for adding into an existing matrix for efficiency.
      
      @param b matrix to add this to. b is not effected
      @return M  A newly created matrix s.t. M = this + b.
      @exception DimensionMismatchException if this and b do not match in dimension.
      @see Matrix#add(Matrix,Matrix,Matrix).
      */
    public Matrix add(Matrix b) throws DimensionMismatchException
    {
      Matrix out = new Matrix(b.rows,
			      b.cols);
      add(this,b,out);
      return out;
    }

    /**
      element-wise matrix subtraction.
      <B>SAFE:</B>  Does not effect either this or b.
      There is static sub method for subtracting into an existing matrix for efficiency.

      @param b matrix to subtract  from this. b is not effected
      @return M  A newly created matrix s.t. M = this - b.
      @exception DimensionMismatchException if this and b do not match in dimension.

      @see Matrix#add(Matrix,Matrix,Matrix).
      */
    public Matrix sub(Matrix b) throws DimensionMismatchException
    {
      Matrix out = new Matrix(b.rows,
			      b.cols);
      sub(this,b,out);
      return out;
    }
  
    /**
      Left-multiply the vector and this matrix, T, and return the result. Does not effect this or a.
      @param a  the vector to premultiply.
      @return v where v = a T.  v will be a new vector.
      @exception DimensionMismatchException if this and a do not match in dimension.

      
      @see Matrix#mult(Vec,Matrix,Vec)
     */
    public Vec preMult(Vec a) throws DimensionMismatchException
    {
      Vec out = new Vec(cols);
      mult(a, this, out);
      return out;
    }

    /**
      Right-multiply the vector and this matrix, T, and return the result.
      Does not effect this or a.
      
      @param a  the vector to post-multiply.
      @return v where v = Ta.  v will be a new vector.
      @exception DimensionMismatchException if this and a do not match in dimension.
	    
      @see Matrix#mult(Matrix,Vec,Vec)      
    */
    public Vec postMult(Vec a) throws DimensionMismatchException
    {
      Vec out = new Vec(rows);
      mult(this, a, out);
      return out;
    }

    /**
      Premultiply A to this matrix, T.
      Does not effect this or A.

      @param A the matrix to premultiply
      @return M s.t. M = AT.  M will be a new matrix.
      @exception DimensionMismatchException if this and A do not match in dimension.
      
      @see Matrix#mult(Matrix,Matrix,Matrix)
      */
    public Matrix preMult(Matrix A) throws DimensionMismatchException
    {
      Matrix out = new Matrix(A.rows, 
			      this.cols);
      mult(A, this, out);
      return out;
    }

    /**
      Postmultiply A to this matrix, T. 
      Does not effect this or A.

      
      @param A the matrix to multiply to this
      @return M s.t. M = TA.  M will be a new matrix.
      @exception DimensionMismatchException if this and A do not match in dimension.

      @see Matrix#mult(Matrix,Matrix,Matrix)
      */
    public Matrix postMult(Matrix A) throws DimensionMismatchException
    {
      Matrix out = new Matrix(this.rows,
			      A.cols);
      mult(this, A, out);
      return out;
    }


    /*
    static { System.loadLibrary("nativeMath"); }
    
    public native int nativeDecomposeLU(int n, int cols, 
					double[][] rep,
					double[][] m, double[] vv, int[] indx);
    
    */

    /**
      Perform an LU decomposition on this and return it as an LUDState object.
      If the matrix is singular, throws SingularMatrixException.
      If the matrix is non-square, returns null and reports error to System.err.
      <B>SAFE:</B> Leaves this matrix unchanged.
      @exception SingularMatrixException if this is singular.
      */
    public void decomposeLU() throws SingularMatrixException
    {
      /* Stolen  from Numerical Recipes in C, Java-ized */
      
      if (lud == null)
	lud = new LUDState(this);

      int n = rows;
      if (n != cols)
	  error("decomposeLU(): can't LU decompose non-square matrix!");


      double[][] m = lud.lud.rep;
      double[] vv = lud.scratchColumn.rep;
      int[] indx = lud.permutation;


      /*
      if (true)
	{
	  lud.d = nativeDecomposeLU(n, cols, rep, m, vv, indx);

	  return;
	}
      */

      double d = 1.0;
      int imax = 0;
      double big, dum, sum, temp;

      int i, j, k;

      for (i=0;i<n;i++) {
	big=0.0;
	for (j=0;j<n;j++)
	  {
	    m[i][j] = rep[i][j];  // copy along the way
	    if ((temp=Math.abs(m[i][j])) > big) big=temp;
	  }
	if (big == 0.0) 
	  {
	    // error("Singular matrix in decomposeLU.  Use SVD.  Punting.");
	    throw new SingularMatrixException();
	    // return null;
	  }
	vv[i] = 1.0/big;
      }
      for (j=0;j<n;j++) {
	for (i=0;i<j;i++) {
	  sum=m[i][j];
	  for (k=0;k<i;k++) sum -= m[i][k] * m[k][j];
	  m[i][j]=sum;
	}
	big=0.0;
	for (i=j; i<n; i++) {
	  sum=m[i][j];
	  for (k=0;k<j;k++)
	    sum -= m[i][k] * m[k][j];
	  m[i][j]=sum;
	  if ( (dum = vv[i] * Math.abs(sum)) >= big) 
	    {
	      big=dum;
	      imax=i;
	    }
	}
	if (j != imax) 
	  {
	    for (k=0;k<n;k++) 
	      {
		dum = m[imax][k];
		m[imax][k] = m[j][k];
		m[j][k] = dum;
	      }
	    d = -d;
	    vv[imax] = vv[j];
	  }
	indx[j] = imax;
	if (m[j][j] == 0.0) 
	  m[j][j] = TINY;
      
	if (j != n-1) 
	  {
	    dum = 1.0/(m[j][j]);
	    for (i=j+1;i<n;i++) 
	      m[i][j] *= dum;
	  }
      }
    
      lud.d = (int) d;
    }
   
    /**
      Calculate the inverse of a square matrix and return it.
      <B>SAFE:</B> does not mutate this.

      @exception SingularMatrixException if singular matrix.
      @exception NonSquareMatrixException if non square
      */
    public Matrix inverse() throws SingularMatrixException, NonSquareMatrixException
    {
      Matrix inv = new Matrix(this);
      this.inverse(inv);
      return inv;      
    }

    /**
      Calculate the inverse of a square matrix and return it in argument.
      <B>SAFE:</B> does not mutate this.
      @param inv the matrix to fill the inverse into.
      @exception SingularMatrixException if singular matrix.
      @exception NonSquareMatrixException if not square.
      */
    public void inverse(Matrix inv) 
	 throws SingularMatrixException, 
	   NonSquareMatrixException
    {
      if (rows != cols)
	{
	  // error("Can't invert non-square matrix.");
	  throw new NonSquareMatrixException("can't invert non square matrix");
	}

      decomposeLU();

      Vec col = lud.scratchColumn;

      int i,j;
      for(j=0; j<rows; j++)
	{
	  for(i=0; i<rows; i++)
	    col.rep[i] = 0.0;
	  col.rep[j] = 1.0;
	
	  lud.backSub(col, col);
	
	  for(i=0; i<rows; i++) 
	    inv.rep[i][j] = col.rep[i];
	}
    }
  
    /*********************
    Statics
    ********************/
    
    /**
      Return the transpose of the matrix.
      <B>SAFE</B> Does not effect old.
      
      @param old the matrix to transpose
      @return a new matrix which is the transpose of the matrix old.
      */
    public static Matrix transpose(Matrix old)
    {
      Matrix m = new Matrix(old);
      m.transpose();
      return m;
    }
  
    /**
      Adds matrices a and b into out. All arguments <em>must</em> have the same dimensions, else out is invalid.
      <B>SAFE</B> for out == a or out == b.
      
      @param a first matrix
      @param b second matrix
      @param out User-allocated storage for a + b.
      @exception DimensionMismatchException if args do not match in dimension.
      
    */
    public static void add(Matrix a, Matrix b, Matrix out) throws DimensionMismatchException
    {
      if (!checkDims(a,b,out))
	{
	  // error("add: can't add dissimilar matrices");
	  throw new DimensionMismatchException();
	}
    
      int i,j;
      for(i=0; i < a.rows; i++)
	for(j=0; j < a.cols; j++)
	  out.rep[i][j] = a.rep[i][j] + b.rep[i][j];
    }
  

    /**
      Subtracts matrices s.t.  out = a -  b.
      All arguments must have the same dimensions, else out is invalid.
      <B>SAFE</B> for out == a or out == b.

      @param a first matrix
      @param b second matrix
      @param out User-allocated object for a - b
      @exception DimensionMismatchException if args do not match in dimension.
      */
    public static void sub(Matrix a, Matrix b, Matrix out) throws DimensionMismatchException
    {
      if (!checkDims(a,b,out))
	{
	  // error("sub: can't add dissimilar matrices");
	  throw new DimensionMismatchException();
	}

      int i,j;
      for(i=0; i < a.rows; i++)
	for(j=0; j < a.cols; j++)
	  out.rep[i][j] = a.rep[i][j] - b.rep[i][j];
    }

    /**
      Multiplies matrices elementwise s.t.  out = a .* b.
      All arguments must have the same dimensions, else out is invalid.
      <B>SAFE</B> for out == a or out == b.

      @param a first matrix
      @param b second matrix
      @param out User-allocated object for a .* b
      @exception DimensionMismatchException if args do not match in dimension.
      */
    public static void elemMult(Matrix a, Matrix b, Matrix out) throws DimensionMismatchException
    {
      if (!checkDims(a,b,out))
	{
	  // error("elemMult: can't element-wise multiply dissimilar matrices");
	  throw new DimensionMismatchException();
	}

      int i,j;
      for(i=0; i < a.rows; i++)
	for(j=0; j < a.cols; j++)
	  out.rep[i][j] = a.rep[i][j] * b.rep[i][j];
    }

  
    /**
      Left multiplies the vector (implicitly assumes a row vector) by the matrix and
      returns result in out.
      <P>
      <B>IN-PLACE MULTIPLY UNSAFE:</B> out <em>cannot</em> be the same object as v!  
         
      @param v vector to left multiply
      @param m matrix to multiply by
      @param out vector where result is stored.  out = v m
      @exception DimensionMismatchException if args do not match in dimension.
      @exception InPlaceMatrixMultException if out is one of other args.
      */
    public static void mult(Vec a, Matrix b, Vec out) throws DimensionMismatchException, InPlaceMatrixMultException
    {
      if (a == out)
	{
	  throw new InPlaceMatrixMultException("mult(Vec a, Matrix b, Vec out): out cannot be a!  Punting.");
	}
      
      if (a.d != b.rows ||
	  out.d != b.cols)
      {
	throw new DimensionMismatchException(
	  "mult(Vec a, Matrix b, Vec out): bad dimensions: " +
	  "1x" + a.d + " * " +
	  b.rows + "x" + b.cols +
	  " != " + "1x" + out.d);
      }
      
      int i,k; 
      for (i=0; i<out.d; i++)
      {
	out.rep[i] = 0.0;
	for(k=0; k < a.d; k++)
	  out.rep[i] += a.rep[k] * b.rep[k][i];
      }
    }
    
    /**
      Right multiplies the vector (implicitly assumes a column vector) by the matrix and
      returns result in out.
      <B>IN-PLACE MULTIPLY UNSAFE:</B> out <em>cannot</em> be the same object as v!  
      
      @param m matrix
      @param v vector
      @param out vector where result is stored.  out = m a
      @exception DimensionMismatchException if args do not match in dimension.
      @exception InPlaceMatrixMultException if out = b
      
      */
    public static void mult(Matrix a, Vec b, Vec out) throws DimensionMismatchException, InPlaceMatrixMultException
    {
      if (b == out)
	{
	  throw new InPlaceMatrixMultException("mult(Matrix a, Vec b, Vec out): can't multiply in place! Punting.");
	}

      if (a.cols != b.d ||
	  out.d != a.rows)
	{
	  throw new DimensionMismatchException(
	    "mult(Matrix a, Vec b, Vec out): bad dimensions: " +
	    a.rows + "x" + a.cols +
	    " * " + b.d + "x1" +
	    " != " + out.d + "x1");
	}
    
      int i,k; 
      for (i=0; i<out.d; i++)
	{
	  out.rep[i] = 0.0;
	  for(k=0; k<b.d; k++)
	    out.rep[i] += a.rep[i][k]*b.rep[k];
	}
    }
    
    /**
      Multiplies the matrix together and stores result in out.
      
      @param a matrix
      @param b matrix
      @param out matrix where result is stored.  out = a b
      @exception DimensionMismatchException if args do not match in dimension.
      @exception InPlaceMatrixMultException if out == a || out == b
      */
    public static void mult(Matrix a, Matrix b, Matrix out) throws DimensionMismatchException, InPlaceMatrixMultException
    {
      if (a == out || b == out)
	{
	  throw new InPlaceMatrixMultException("mult(Matrix a, Matrix b, Matrix c): can't multiply in place!");
	}
    
      if (a.cols != b.rows ||
	  out.rows != a.rows ||
	  out.cols != b.cols)
      {
	throw new DimensionMismatchException("mult(Matrix A, Matrix B, " +
					     "Matrix out): bad dimensions on " +
					     "matrix multiply. A is " +
					     a.rows + "x" + a.cols +
					     ", B is " + b.rows + "x" +
					     b.cols + ", out is " +
					     out.rows + "x" + out.cols);
      }
    
      int i,j,k; // i E [0 .. a.rows-1]  j E [0 .. b.cols-1]  k E [0 .. a.cols-1]
      for (i=0; i<a.rows; i++)
	for(j=0; j<b.cols; j++)
	  {
	    out.rep[i][j] = 0.0;
	    for(k=0; k<a.cols; k++)
	      out.rep[i][j] += a.rep[i][k] * b.rep[k][j];
	  }
    }
  
    transient LUDState lud;

    /**
      Calculate the determinant.
      */
    public double determinant() throws NonSquareMatrixException, SingularMatrixException
    {
      if (rows != cols)
	{
	  throw new NonSquareMatrixException("Can't compute determinant of non-square matrix.");
	}

      decomposeLU();

      return lud.determinant();
    }

    /** 
	Get the determinant after calculating the inverse.<p>
	Use this after computing the inverse to skip an unnecessary
	LU decomposition.
    */
    public double determinantAfterInverse() throws SingularMatrixException
    {
      if (lud == null)
	decomposeLU();
      
      return lud.determinant();
    }


    /** 
	Return true if the argument is a Matrix of the same size
	and values as this Matrix. 
    */
    public boolean equals(Object obj)
    {
      try 
	{
	  Matrix m = (Matrix) obj;
	  
	  int i,j;
	  for (i = 0; i < rows; i++)
	    for (j = 0; j < cols; j++)
	      if (rep[i][j] != m.rep[i][j])
		return false;
	  if (rows != m.rows) return false;
	  if (cols != m.cols) return false;
	  
	  return true;
	}
      catch (Exception e)
	{
	  return false;
	}
    }

    /**
       Return a hash key for this Matrix based on its values.
    */
    public int hashCode()
    {
      double sum = 0;
      if (rows == 0)
	return 0;
      int n;
      if (rows < cols)
	n = rows;
      else
	n = cols;
      for (int i = 0; i < n; i++)
	sum += rep[i][i];

      long v = Double.doubleToLongBits(sum);
      return (int) (v^(v>>>32));

      /*
	// this is just too slow:
      // compute the XOR of the two halves of each matrix value,
      // and XOR the result over all values.
      // this is O(N^2); perhaps better to do over trace (O(N)).
      long l = 0, v;
      for (int i = 0; i < rows; i++)
	for (int j = 0; j < cols; j++)
	  {
	    v = Double.doubleToLongBits(rep[i][j]);
	    l = l^(v^(v>>>32));
	  }
      return (int) l;
      */
    }


    /****************************
    Amyl Nitrate available below! (cyanide)
    ***************************/ 
    protected static void error(String s)
    {
      System.err.println("(Matrix): " + s);
    }

    protected static boolean checkDims(Matrix a, 
				       Matrix b,
				       Matrix out)
    {
      return (a.rows == b.rows &&
	      a.rows == out.rows &&
	      a.cols == b.cols &&
	      a.cols == out.cols);
    }


    protected static double[][] copyRep(double[][] a)
    {
      // make rows array
      double[][] v = new double[a.length][];
      int i;
      for (i=0; i<a.length; i++)
	{
	  v[i] = Vec.copyArray(a[i]);
	}
      return v;
    }
 
  
    protected static void print(String s)
    {
      System.out.println(s);
    }

    /**********************
    Chuck Yeager
    *********************/
    
    /** Test driver for the class */
    /*
    public static void main(String[] argh)
    {
      Matrix a = new Matrix (3, 3);
      Matrix b = new Matrix (3, 4);
    
      int i, j;
      for(i=0; i < 3; i++)
	for(j=0; j < 3; j++)
	  a.set(i,j,2*i+j);
    
      for(i=0; i < 3; i++)
	for(j=0; j < 4; j++)
	  b.set(i,j,-2*i+j);
    
      print("a: " + a);
      print("b: " + b);
      b.transpose();
      print("b trans: " + b);
      b.transpose();
    
      Matrix c = null;
      try
	{
	  c = a.preMult(b);
	}
      catch (DimensionMismatchException e)
	{
	  System.out.println("Expected dimension mismatch and got it.");
	}

      c = a.postMult(b);
    
      print("c = ab: " + c);

      Matrix id = new Matrix (4,4);
      id.identity();
      print("identity: " + id);

      Matrix r1 = makeRandomMatrix(6);
      Matrix r1inv = null;
      try
	{
	  r1inv = r1.inverse();
	}
      catch (SingularMatrixException e)
	{
	  System.err.println("Matrix " + r1 + " is singular.  Can't invert.");
	  System.exit(-1);
	}
      catch (NonSquareMatrixException e)
	{
	  System.err.println("Can't invert non-square.  Implement pseduo-inverse");
	  System.exit(-1);
	}
      Matrix id1 = r1.postMult(r1inv);
      id1.roundZeros();
      print("r1: " + r1);
      print("r1 * r1 inverse: " + id1);
    
      
      Matrix r2 = makeRandomMatrix(7);
      Matrix r2inv = null;
      try
	{
	  r2inv = r2.inverse();
	}
      catch (SingularMatrixException e)
	{
	  System.err.println("Matrix " + r2 + " is singular.  Can't invert.");
	  System.exit(-1);
	}
      catch (NonSquareMatrixException e)
	{
	  System.err.println("Can't invert non-square.  Implement pseduo-inverse");
	  System.exit(-1);
	}
      Matrix id2 = r2.preMult(r2inv);
      id2.roundZeros();
      print("r2: " + r2);
      print("r2inv * r2: " + id2);
        
    }
    */
    /**
       create an n x n matrix of random doubles in the range [-1, 1]
    */
    /*
    public static Matrix makeRandomMatrix(int n)
    {
      return Matrix.makeRandomMatrix(n, -1.0, 1.0);
    }
    */

    /** 
	Create an n by n matrix of random double in the range [lo, hi]
     */
    /*
    public static Matrix makeRandomMatrix(int n, double lo, double hi)
    {
      Matrix m = new Matrix(n,n);
      UniformRandomVariable urv = new UniformRandomVariable(lo, hi);
      int i,j;
      for(i=0; i<n; i++)
	for(j=0; j<n; j++)
	  m.set(i,j,urv.sample());
      return m;
    }
    */
  }
